/***********************
Script: ajaxlivesearch.js
version: 1.0 Beta
Purpose: Create ajax-powered user-friendly and interactive searches
Author : Frédéric Leroy (aka Fly06)
Copyright : copyright (c) 2011 Fly06.Fr, <http://www.fly06.fr>
License: GNU/GPL
Javascript API : MooTools v1.2
MooTools More Requirements : Hash.Cookie 
------------------------
This file contains 3 classes:
- ajaxLiveSearch: get and show the total number of results only
- ajaxLiveSearchWithResults: ajaxLiveSearch stuff PLUS show the x first results links
- ajaxLiveSearchWithResultsAndNav: ajaxLiveSearchWithResults stuff PLUS ability to navigate among all results links
***********************/

var ajaxLiveSearch = new Class({
	Implements: [Options, Events],
	// Options
	options: {
		// Caching
		doCaching: true, // bool
		cachingCookieName: 'ajaxLiveSearchCookie', // string
		cachingCookieOptions: {}, // hash (eg: {duration: 3600})
		// Input Search Box
		defaultSearchText: 'Search...', // string
		// Messages
		msgServerNoResultFound: 'No result found!', // string
		msgServerOneResultFound: 'Only 1 result found', // string
		msgServerSeveralResultsFound: '{total} results found', // string
		msgServerError: {
			1: 'No keyword found!',
			2: 'Search string too short/long!',
			3: 'Invalid keyword(s)!',
		},
		msgServerErrorUnknown: 'Unknown error!', // string
		// Events
		onShowMsg: $empty, // function
		onHideMsg: $empty, // function
		onSetCache: $empty, // function
		onGetCache: $empty // function
	},
	// Properties - Configuration
	searchElt: null, // imput box id
	msgElt: null, // msg block id
	loadingElt: null, // loading block id
	baseUrl: null, // url to request on the server 
	// Properties - Private
	cacheStore: null, // Hash or Hash.Cookie object
	ajaxRequest: null, // Request.JSON object
	// Initialization Method
	initialize: function(searchElt, msgElt, loadingElt, baseUrl, options) {
		// Merge the options
		this.setOptions(options);
		// Make elements
		this.searchElt = $(searchElt);
		this.msgElt = $(msgElt);
		this.loadingElt = $(loadingElt);
		// Set Base URL (Ajax)
		this.baseUrl = baseUrl;
		// Ajax Request Object
		this.ajaxRequest = this.createAjaxRequest();
		// Force defaultSearchText
		if (this.options.defaultSearchText) {
			this.searchElt.setProperty('value', this.options.defaultSearchText);
		}
		// Init cache store
		this.cacheStore = this.makeCacheStore();
		// input box - blur event handler
		this.makeSearchBlurEventHandler();		
		// input box - focus event handler
		this.makeSearchFocusEventHandler();	
		// input box - keyup event handler
		this.makeSearchKeyupEventHandler();
	},
	// Make Cache Store
	makeCacheStore: function() {
		if (this.options.doCaching) {
			return new Hash.Cookie(this.options.cachingCookieName, this.options.cachingCookieOptions);
		} else {
			return null;
		}
	},
	// Event Handling Methods
	makeSearchBlurEventHandler: function() {
		this.searchElt.addEvent('blur', function() {
			if(this.searchElt.value == '') {
				this.searchElt.value = this.options.defaultSearchText;
			}
		}.bind(this));
	},
	makeSearchFocusEventHandler: function() {
		this.searchElt.addEvent('focus', function() {
			if(this.searchElt.value == this.options.defaultSearchText) this.searchElt.value = '';
		}.bind(this));
	},
	makeSearchKeyupEventHandler: function() {	
		this.searchElt.addEvent('keyup', function(event) {
			// We cancel the previous ajax call
			this.ajaxRequest.cancel();
			// Enter key
			if(event.key == "enter") {
				this.hide();
				return;
			}
			// Arrow keys
			if((event.key == "up") || (event.key == "down") || (event.key == "left") || (event.key == "right")) {
				return;
			}
			// Visual cue to show that the search is running in the background
			this.showLoading();
			// Format the searchword
			var searchText = this.format(this.searchElt.get('value'));
			// The string is empty
			if (!searchText) {	
				this.hide();
				return;
			}
			// No need to make the ajax call in this case
			if (this.options.doCaching) {
				this.cacheStore.each(function(objSearch, key) {
					if (objSearch.value == 0 && searchText.test(key.escapeRegExp())) {
						this.fireEvent('onGetCache', [searchText, objSearch]);	
						this.show(objSearch);
						return;					
					}
				}, this);	
			}	
			// Already searched and cached
			if (this.options.doCaching) {
				if ($defined(this.cacheStore.get(searchText))) {
					var objSearch = this.cacheStore.get(searchText);
					this.fireEvent('onGetCache', [searchText, objSearch]);					
					this.show(objSearch);
					return;
				} 
			}			
			// Ok, let's get the search results from the server 
			this.ajaxRequestSend(searchText);
		}.bind(this));
	},
	// Ajax Request Methods
	createAjaxRequest: function() {
		var request = new Request ({
			url: this.baseUrl,
			data: this.ajaxRequestData,
			noCache: true,
			onSuccess : function(response) {
				this.ajaxRequestResponse(response);
			}.bind(this)	
		}, this);
		return request;
	},
	ajaxRequestResponse: function(response) {
		jsonObj = JSON.decode(response);
		// Store the (searchword, obj) pair in the cache
		var objSearch = jsonObj.objSearch;
		if(this.options.doCaching) {			
			this.fireEvent('onSetCache', [jsonObj.searchword, objSearch]);	
			this.cacheStore.set(jsonObj.searchword, objSearch);		
		}
		// Show the returned info
		this.show(objSearch);
	},
	ajaxRequestSend: function(searchText) {
		this.ajaxRequest.send({'data' : {'searchword' : searchText}});
	},
	// Elements Hide/Show Methods
	show: function(objSearch) {
		this.hide(this.loadingElt);
		this.showMsg(objSearch.type, objSearch.value);
	},
	hide: function() {
		var slice = Array.prototype.slice;
		var args = slice.apply(arguments);
		args = (args.length ? args : args.extend([this.msgElt, this.loadingElt]));
		args.each(function(object) {
			if (object == this.msgElt) this.fireEvent('onHideMsg');	
			object.hide();
		}, this);
	},
	showMsg: function(type, value) {
		var msg = this.makeMsg(type, value);
		this.fireEvent('onShowMsg', [type, value, msg]);	
		this.msgElt.empty().set('html', msg);
		this.msgElt.show()	
	},
	showLoading: function() {
		this.hide(this.msgElt);
		this.loadingElt.show();
	},
	// Format the search text
	format: function(searchText) {
		searchText = searchText.trim();
		if (searchText) {
			var arrSearchText = searchText.split(' ');
			arrSearchText = arrSearchText.map(function(val){
    				return val.toString().trim().toLowerCase();
			});
			arrSearchText.combine([]); // unique() method workaround
			arrSearchText.sort();
			searchText = arrSearchText.join(' ');
		}
		return searchText;
	},
	// Messages Creation Methods
	makeMsg: function(type, value) {
		return eval('this.make'+type.capitalize()+'Msg(value)');
	},
	makeCountMsg: function(count) {
		switch (count) {
			case 0:
				return this.options.msgServerNoResultFound;
				break;
			case 1:
				return this.options.msgServerOneResultFound;
				break;
			default:
				var myObj = {'total' : count};
				var myString = this.options.msgServerSeveralResultsFound;
				return myString.substitute(myObj);
				break;				
		}
	},
	makeErrorMsg: function(errnum) {
		var errmsg = this.options.msgServerError[errnum];
		return ($defined(errmsg) ? errmsg : this.options.msgServerErrorUnknown);
	}
});

var ajaxLiveSearchWithResults = new Class({
	Extends: ajaxLiveSearch,
	// Options
	options: {
		// Message
		msgResultsCounter: 'Results: {from}-{to}', // string
		// Results
		limitResults: 5, // int
	},
	// Properties - Configuration
	resultsElt: null, // results list element
	resultsCounterElt: null, // results counter element
	// Initialization Method
	initialize: function(resultsCounterElt, resultsElt, searchElt, msgElt, loadingElt, baseUrl, options) {
		// Parent initialization
		this.parent(searchElt, msgElt, loadingElt, baseUrl, options);
		// Merge the options
		this.setOptions(options);
		// Make elements
		this.resultsElt = $(resultsElt);
		this.resultsCounterElt = $(resultsCounterElt);	
	},
	ajaxRequestSend: function(searchText) {
		var data = {'searchword' : searchText, 'show_results': 1, 'limit_results': this.options.limitResults};
		this.ajaxRequest.send({'data' : data});
	},
	// Show results
	show: function(objSearch) {
		// We call the show method of the parent class
		this.parent(objSearch);
		if (objSearch.type == 'count') {
			if ($defined(objSearch.results) && objSearch.results.length) {
				var count_results = Math.min(objSearch.results.length, this.options.limitResults);
				var results = objSearch.results.slice(0, count_results);
				var list_elements = this.makeAll(results);
				list_elements.each(function(list_element) {
					this.resultsElt.grab(list_element);
				}, this);
				this.resultsElt.show();
				this.makeCounter(1, count_results);
				this.resultsCounterElt.show();	
			}
		}
	},
	// Hide results element
	hide: function() {
		this.parent();
		this.resultsElt.empty().hide();
		this.resultsCounterElt.empty().hide();
	},
	// Make Cache Store
	makeCacheStore: function() {
		var obj = null;
		if (this.options.doCaching) {
			obj = new Hash({});
		}
		return obj;
	},
	// Html Results Creation Methods
	makeAll: function(results) {
		var list_elements = [];
		results.each(function(result) {
			list_elements.include(this.makeOne(result));
		}, this);
		return list_elements;
	},
	makeOne: function(result) {
		var list_element = new Element('li');
		var link = new Element('a', {
				'href': result.href,
				'title': result.text,
				'html': result.title
		});
		list_element.adopt(link);
		return list_element;
	},
	// Make/update results counter element
	makeCounter: function(from, to) {
		var myObj = {'from': from, 'to' : to};
		var html = this.options.msgResultsCounter.substitute(myObj);
		this.resultsCounterElt.set('html', html);
	}
});

var ajaxLiveSearchWithResultsAndNav = new Class({
	Extends: ajaxLiveSearchWithResults,
	// Options
	options: {
		// Button Event
		buttonEvent: 'click',
		// Events
		onShowNav: $empty, // function
		onHideNav: $empty // function
	},
	// Properties - Configuration
	nextButtonElt: null,
	prevButtonElt: null,
	// Properties - Private
	currentResults: null,
	initialize: function(resultsCounterElt, nextButtonElt, prevButtonElt, resultsElt, searchElt, msgElt, loadingElt, baseUrl, options) {
		// Parent initialization
		this.parent(resultsCounterElt, resultsElt, searchElt, msgElt, loadingElt, baseUrl, options);
		// Properties - Configuration	
		this.nextButtonElt = $(nextButtonElt);	
		this.prevButtonElt = $(prevButtonElt);
		// nextButtonElt event handler
		this.makeNextEventHandler();
		// prevButtonElt event handler
		this.makePrevEventHandler();	
	},
	ajaxRequestSend: function(searchText) {
		var data = {'searchword' : searchText, 'show_results': 1, 'limit_results': 0};
		this.ajaxRequest.send({'data' : data});
	},
	makeNextEventHandler: function() {
		this.nextButtonElt.addEvent(this.options.buttonEvent, function(e) {
			if (e) e.stop();
			// Check
			var nindex = this.nextButtonElt.retrieve('index');
			if (nindex == this.currentResults.length) {
				return;
			} 
			// Results list update
			var result = this.currentResults[nindex];
			var list_element = this.makeOne(result);
			list_element.inject(this.resultsElt, 'bottom');
			this.resultsElt.getFirst('li').destroy();
			// Navigation update
			nindex++;
			this.makeCounter(nindex - this.options.limitResults + 1, nindex);
			this.nextButtonElt.store('index', nindex);
			if (nindex == this.currentResults.length) {
				this.nextButtonElt.setStyle('visibility', 'hidden');			
			} 
			var pindex = this.prevButtonElt.retrieve('index');
			pindex++;
			this.prevButtonElt.store('index', pindex);
			if (this.prevButtonElt.getStyle('visibility') == 'hidden') {
				this.prevButtonElt.setStyle('visibility', 'visible');
			}
		}.bind(this));
	},
	makePrevEventHandler: function() {
		this.prevButtonElt.addEvent(this.options.buttonEvent, function(e) {
			if (e) e.stop();
			// Check
			var pindex = this.prevButtonElt.retrieve('index');
			if (pindex == -1) {
				return;
			} 
			// Results list update
			var result = this.currentResults[pindex];
			var list_element = this.makeOne(result);
			list_element.inject(this.resultsElt, 'top');
			this.resultsElt.getLast('li').destroy();
			// Navigation update
			this.makeCounter(pindex + 1, pindex + this.options.limitResults);
			pindex--;
			this.prevButtonElt.store('index', pindex);
			if (pindex == -1) {
				this.prevButtonElt.setStyle('visibility', 'hidden');
			} 
			var nindex = this.nextButtonElt.retrieve('index');
			nindex--;
			this.nextButtonElt.store('index', nindex);
			if (this.nextButtonElt.getStyle('visibility') == 'hidden') {
				this.nextButtonElt.setStyle('visibility', 'visible');
			}
		}.bind(this));
	},
	show: function(objSearch) {
		// We call the show method of the parent class
		this.parent(objSearch);
		if ($defined(objSearch.results) && (objSearch.results.length > this.options.limitResults)) {
			this.fireEvent('onShowNav');
			// We save the current results
			this.currentResults = objSearch.results;
			// We initialize the next/prev buttons
			this.nextButtonElt.store('index', this.options.limitResults);
			this.prevButtonElt.store('index', -1);
			// We show the above elements
			this.nextButtonElt.show();
			this.prevButtonElt.show('hidden');
		}
	},
	hide: function() {
		this.parent();
		this.fireEvent('onHideNav');
		this.nextButtonElt.hide();
		this.prevButtonElt.hide();
	}
});

/***********************
Element override
***********************/

Element.implement({
	'hide': function(){ 
		this.setStyle('display', 'none');
		var visibility = ($defined(arguments[0]) ? arguments[0] : 'hidden');
		this.setStyle('visibility', visibility);
	},
	'show': function(){ 
		this.setStyle('display', 'block');
		var visibility = ($defined(arguments[0]) ? arguments[0] : 'visible');
		this.setStyle('visibility', visibility);
	}
});
